import requests as _requests
import os
import pandas as pd
from datetime import datetime
import time
import dotenv


# I think I would need a server to have alphavantagekey or users can supply their own
dotenv.load_dotenv('./.env')
key = os.environ.get('ALPHA_VANTAGE_KEY')  # os.getenv()


def _get_json(url: str):
    request = _requests.get(url)
    json = request.json()
    return json


class Ticker:
    def __init__(self, ticker):
        self.ticker = ticker.upper()

    def get_data(self):
        # Quicker version? url = 'https://www.alphavantage.co/query?function=GLOBAL_QUOTE&symbol=IBM&apikey=demo'
        # Can access more data but will cost a lot of time
        json = _get_json(
            f'https://www.alphavantage.co/query?function=TIME_SERIES_DAILY_ADJUSTED&symbol={self.ticker}&apikey={key}')
        while not json:
            try:
                json = _get_json(
                    f'https://www.alphavantage.co/query?function=TIME_SERIES_DAILY_ADJUSTED&symbol={self.ticker}&apikey={key}')['Time Series (Daily)']
                break
            except:
                time.sleep(3)

        # data = json['Time Series (Daily)']
        df = pd.DataFrame([[datetime.strptime(date, '%Y-%m-%d'), json[date]['4. close'], json[date]['2. high'], json[date]['3. low'], json[date]['1. open'], json[date]['6. volume']] for date in json.keys()],
                          columns=['Date', 'Close', 'High', 'Low', 'Open', 'Volume'])
        df = df.set_index('Date')

        return df.iloc[:90]

    # maybe do &outputsize=full, idk what to use for.
    def get_intra_day_data(self):
        url = f'https://www.alphavantage.co/query?function=TIME_SERIES_INTRADAY&symbol={self.symbol}&interval=5min&apikey={key}'
        json = _get_json(url)
        while not json:
            try:
                json = _get_json(url)
                break
            except:
                time.sleep(3)

        df = pd.DataFrame([[datetime.strptime(date, '%Y-%m-%d'), json[date]['4. close'], json[date]['2. high'], json[date]['3. low'], json[date]['1. open'], json[date]['6. volume']] for date in json.keys()],
                          columns=['Date', 'Close', 'High', 'Low', 'Open', 'Volume'])
        df = df.set_index('Date')
        return df

    def get_company_info(self):
        url = f'https://www.alphavantage.co/query?function=OVERVIEW&symbol={self.symbol}&apikey={key}'
        json = _get_json(url)
        while not json:
            try:
                json = _get_json(url)
                break
            except:
                time.sleep(3)

        df = pd.DataFrame([[datetime.strptime(date, '%Y-%m-%d'), json[date]['4. close'], json[date]['2. high'], json[date]['3. low'], json[date]['1. open'], json[date]['6. volume']] for date in json.keys()],
                          columns=['Date', 'Close', 'High', 'Low', 'Open', 'Volume'])
        df = df.set_index('Date')
        return df



from datetime import date, datetime
from .base import Ticker
import pandas as pd 
from requests import get
from bs4 import BeautifulSoup
import re
from GoogleNews import GoogleNews
from dotenv import load_dotenv
import os 
import tweepy

# When I have time, replace yahoo api for alphavantage. https://rapidapi.com/alphavantage/api/alpha-vantage, https://www.alphavantage.co/premium/. https://rapidapi.com/mpeng/api/stock-and-options-trading-data-provider
# I can access a lot more stuff (like news and related) by directly accessing API. Most of these Stock. methods can be replaced by accessing API lol.

class Portfolio:
    # Maybe do a DataFrame? Easier to display, can sort. Columns for tickers, change_percentage, last_updated_price
    # Pass in stocks with their target values, like this: (stock, target_prices)
    def __init__(self, stocks, interval='1m', period='1d'):
        # if type(stocks) == tuple:
        # 	self.stocks = stocks[0]

        # if type(stocks[0]) != str:
        # 	while stocks[0] != str:
        # 		stocks = stocks[0]
        # 		print(stocks)
        if type(stocks) == list:
            self.stocks = {Stock(stock, interval=interval, period=period) for stock in sorted(stocks)}
        else:
            self.stocks = {Stock(stock, interval=interval, period=period) for stock in sorted(stocks.split())}	

    def add_stocks(self, stocks):
        for stock in stocks.split():
            self.stocks.add(stock)

    def clean_stocks(self):
        for stock in self.stocks:
            if 'trash' in stock.ticker:
                self.stocks.remove(stock)

    def get_biggest_movers(self):
        print("These are today's biggest movers: ")
        for stock in self.stocks:
            if stock.daily_change_percentage > 10.0:
                print(stock.ticker)

    def get_highest_rel_volume(self):
        print("These are today's most relatively active stocks: ")
        for stock in self.stocks:
            if stock.get_relative_volume() > 3.0:
                print(stock.ticker)
        # return [stock.ticker for stock in self.stocks if stock.get_relative_volume() > 3.0]

    def get_stocks(self):
        return sorted([(stock.ticker, stock.daily_stats(), stock.daily_change_percentage()) for stock in self.stocks], key=lambda x: x[0])

    def get_stocks_daily(self): # Fix
        return [stock.df_month for stock in self.stocks]

    def get_stocks_intra(self):
        return [stock.df for stock in self.stocks]

    def sort_by(self, sort='name'):
        if sort == 'name':
            pass
        elif sort == 'change_percentage':
            pass
        elif sort == 'price':
            pass
        else:
            raise Exception('Please enter one of the following sorting methods: name, price, or change_percentage')

    def update_price_change(self):
        print("These stocks' prices have changed significantly.")
        for stock in self.stocks:
            if (stock.df.iloc[0]['Close'] - stock.get_last_updated())/(stock.get_last_updated())*100 > 5.0:
                stock.set_last_updated(stock.df.iloc[0]['Close'])
                print(stock.ticker)

    def __iter__(self):
        return iter(self.stocks)

    def __getitem__(self, i):
        return self.stocks[i]

    def __len__(self):
        return len(self.stocks)

    def __repr__(self):
        return 'A portfolio of #winning stocks.'

class Stock:

    def __init__(self, ticker,  target_prices=None, price_invested=None):
        # interval='1m', period='1d',
        try:
            self.ticker = ticker
            self.df = Ticker(ticker).get_data()
            self.prev_close = self.df.iloc[-1]
            self.target_prices = target_prices
            self.price_invested = price_invested
        except:
            raise Exception('Sorry, we could not find this stock!')
            

    def get_month_data(self):
        # This one should be full then cut down to 24 months
        df = Ticker(self.ticker).get_data()
        df.index = pd.to_datetime(df.index)
        for i in range(2, df.shape[0]):
            df.loc[df.index[i], '2_sma'] = sum([float(i) for i in df.iloc[i-2:i]['Close']])/2
        for i in range(9, df.shape[0]):
            df.loc[df.index[i], '9_sma'] = sum([float(i) for i in df.iloc[i-9:i]['Close']])/9
        try:
            df = df[['Close', 'Open', 'High', 'Low', '2_sma', '9_sma', 'Volume']]
        except:
            pass
        return df

    def add_target_prices(self, new_target_prices):
        self.target_prices = new_target_prices

    def daily_change_percentage(self):
        return ((self.df.iloc[-1]['Close'] - self.df.iloc[-2]['Close'])/self.df.iloc[-2]['Close'])*100

    def daily_high_change_percentage(self):
        return ((self.df.iloc[-1]['High'] - self.df.iloc[-2]['Close'])/self.df.iloc[-2]['Close'])*100

    def daily_stats(self):
        df = Ticker(self.ticker).get_data('1d', '1d')
        df.index = pd.to_datetime(df.index)		
        return self.df.iloc[-1]['Close'], self.df.iloc[-1]['High'], self.df.iloc[-1]['Low']

    def get_relative_volume(self):
        avg_volume = sum(self.df['Volume'])/len(self.df)
        return self.df.iloc[-1]/avg_volume

    # def get_last_updated(self):
    #     return self._last_updated_price

    # def set_last_updated(self, price):
    #     self._last_updated_price = price

    def _find_match(self, pattern, text):
        match = pattern.search(text)
        return match

    def _no_attributes(self, tag):
        if 'td' in str(tag):
            return tag.has_attr('class') or tag.has_attr('id')

    HEADERS = {'User-Agent': "'Mozilla/5.0 (Windows NT 10.0; WOW64; rv:54.0) AppleWebKit/537.36 " # Telling the website what browser I am "using"
                                "(KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36'"}

    def _get_soup(self, url):
        response = get(url, headers=self.HEADERS, timeout=20)
        assert response.status_code == 200
        return BeautifulSoup(response.content, 'lxml')

    # Make this all one big function
    def get_summary(self):
        BASE_URL = f'https://www.marketwatch.com/investing/stock/{self.ticker}'
        soup = self._get_soup(BASE_URL)

        summary = soup.find('p', {'class': 'description__text'})

        BASE_URL = f'https://finance.yahoo.com/quote/{self.ticker}?p={self.ticker}'
        soup = self._get_soup(BASE_URL)
        website = soup.find('a', {'title': 'Company Profile'})
        if website:
            website = website['href']

        return summary.get_text(), website


        # Find the website of the stock and go to its information page

    def basic_stats(self):
        # Market cap, avg volume, financial information
        BASE_URL = f'https://finance.yahoo.com/quote/{self.ticker}/key-statistics?p={self.ticker}'
        soup = self._get_soup(BASE_URL)

        info = []

        # Financial highlights
        div = soup.find('div', {'class': 'Mb(10px) Pend(20px) smartphone_Pend(0px)'})
        info.append([(i.find('h3', {'class': 'Mt(20px)'}), i.find('tbody').find_all('tr')) for i in div.find_all('div', {'class': 'Pos(r) Mt(10px)'})])

        # Trading Information
        div = soup.find('div', {'class': 'Pstart(20px) smartphone_Pstart(0px)'})
        info.append([(i.find('h3', {'class': 'Mt(20px)'}), i.find('tbody').find_all('tr')) for i in div.find_all('div', {'class': 'Pos(r) Mt(10px)'})])

        info_df = pd.DataFrame(info, columns={'Label', 'Value'})

        # Income Statement
        nerd_info = []
        BASE_URL = f'https://finance.yahoo.com/quote/{self.ticker}/financials?p={self.ticker}'
        soup = self._get_soup(BASE_URL)
        rows = soup.find_all('div', {'data-test': 'fin-row'})
        for row in rows:
            title = row.find('div', {'class': 'D(tbc) Ta(start) Pend(15px)--mv2 Pend(10px) Bxz(bb) Py(8px) Bdends(s) Bdbs(s) Bdstarts(s) Bdstartw(1px) Bdbw(1px) Bdendw(1px) Bdc($seperatorColor) Pos(st) Start(0) Bgc($lv2BgColor) fi-row:h_Bgc($hoverBgColor) Pstart(15px)--mv2 Pstart(10px)'})
            data = row.find('div', {'data-test': 'fin-col'})
            nerd_info.append([title.get_text(), data.get_text()])
        
        # Balance Sheet
        BASE_URL = f'https://finance.yahoo.com/quote/{self.ticker}/balance-sheet?p={self.ticker}'
        soup = self._get_soup(BASE_URL)
        rows = soup.find_all('div', {'data-test': 'fin-row'})
        for row in rows:
            title = row.find('D(ib) Fw(b) Ta(start) Px(15px)--mv2 Px(10px) W(247px)--mv2 W(222px) Bxz(bb) Bdendw(1px) Bdstartw(1px) Bdbw(1px) Bdends(s) Bdstarts(s) Bdbs(s) Bdc($seperatorColor) Py(6px) Pos(st) Start(0) Bgc($lv2BgColor)')
            data = row.find('div', {'data-test': 'fin-col'})
            nerd_info.append([title.get_text(), data.get_text()])

        nerd_df = pd.DataFrame(nerd_info, columns={'Label', 'Value'})
        
        return info_df, nerd_df

    def price_target(self, exchange='NASDAQ'): # Automatically find correct stock exchange
        BASE_URL = f'https://www.marketbeat.com/stocks/{exchange}/{self.ticker}/price-target/'
        soup = self._get_soup(BASE_URL)
        
        price_target = soup.find('table', {'class': 'scroll-table'})
        _pattern = re.compile(r'Price Target: \$\d{1,3}\.\d\d')
        price_target = self._find_match(_pattern, table.get_text()).group(0)
        _pattern = re.compile(r'\d{1,3}\.\d\d\% \w{6,8}')
        percentage = self._find_match(_pattern, table.get_text()).group(0)

        BASE_URL = f'https://finviz.com/quote.ashx?t={self.ticker}'
        response = get(BASE_URL, headers=HEADERS, timeout=20)
        soup = BeautifulSoup(response.content, 'lxml')
        table = soup.find('table', {'class': "fullview-ratings-outer"})
        rows = table.find_all('td', {'class': 'fullview-ratings-inner'})
        df_data = []
        for row in rows:
            row = row.find_all('td')
            date, _, fund, action, pricetarget = [val.get_text() for val in row]
            date = datetime.strptime(date, '%b-%d-%y')
            df_data.append((date, fund, action, pricetarget))

        analyst_price_targets = pd.DataFrame(df_data, columns=['Date', 'Fund', 'Action', 'PriceTarget'])
        analyst_price_targets = analyst_price_targets.set_index('Date')
        return price_target, percentage, analyst_price_targets

    def price_predictions(self):
        BASE_URL = f'https://www.barchart.com/stocks/quotes/{self.ticker}/opinion'
        soup = self._get_soup(BASE_URL)

        table = soup.find('table', {'data-ng-class': "{'hide': currentView !== 'strengthDirection'}"})
        titles = soup.find_all('tr', {'class': 'indicator-title'})
        titles = [i.get_text() for i in titles]

        data = soup.find_all('tr', {'class': 'indicator-item'})
        data = [i.get_text() for i in data]
        data = data[len(data)//2 + 1:]
        df_data = []
        for i in data:
            signal, strength, direction = i.split()[-3:]
            indictator = ' '.join(i.split()[:-3])
            df_data.append((indictator, signal, strength, direction))
        df = pd.DataFrame(df_data, columns=['Indictator', 'Signal', 'Strength', 'Direction'])
        return df, titles

    def ta_indictators(self): # Loads wrong page. Beta, RSI history, above/below 9 SMA, above/below 180 SMA, volatility, rel volume, 52W high/low
        # OBV: calculate OBV, then compare to stock close price, say how close they correlate.
  

        pass


    def news_sentiments(self): # Returns news articles curated via Finviz, Yahoo, and Google News, GET UNUSUAL OPTION ACTIVITY
        BASE_URL = f'https://finviz.com/quote.ashx?t={self.ticker}'
        soup = self._get_soup(BASE_URL)

        table = soup.find('table', {'class': 'fullview-news-outer'})
        rows = table.find_all('tr')
        df_data = []
        for row in rows:
            date = row.find('td', {'align': 'right'})
            article = row.find('td', {'align': 'left'})
            link = article.find('a')['href']
            df_data.append((date.get_text(), article.get_text(), link))
        df = pd.DataFrame(df_data, columns=['Time', 'Headline', 'Link'])


        BASE_URL = f'https://finance.yahoo.com/quote/{self.ticker}/news?p={self.ticker}'
        soup = self._get_soup(BASE_URL)

        links = soup.find_all('a', {'class': 'js-content-viewer wafer-caas Fw(b) Fz(18px) Lh(23px) LineClamp(2,46px) Fz(17px)--sm1024 Lh(19px)--sm1024 LineClamp(2,38px)--sm1024 mega-item-header-link Td(n) C(#0078ff):h C(#000) LineClamp(2,46px) LineClamp(2,38px)--sm1024 not-isInStreamVideoEnabled'})
        news = [(link.get_text(), str('yahoo.com' + link['href'])) for link in links]

        BASE_URL = f'https://finance.yahoo.com/quote/{self.ticker}/press-releases?p={self.ticker}'
        soup = self._get_soup(BASE_URL)

        links = soup.find_all('a', {'class': 'js-content-viewer wafer-caas Fw(b) Fz(18px) Lh(23px) LineClamp(2,46px) Fz(17px)--sm1024 Lh(19px)--sm1024 LineClamp(2,38px)--sm1024 mega-item-header-link Td(n) C(#0078ff):h C(#000) LineClamp(2,46px) LineClamp(2,38px)--sm1024 not-isInStreamVideoEnabled'})
        press_releases = [(link.get_text(), str('yahoo.com' + link['href'])) for link in links]
        # Look for keywords in the news? Any showcases, Investor/analyst days, Analyst revisions, Management transitions
        # Product launches, Significant stock buyback changes
  
  
          # Getting news from google news search
        googlenews = GoogleNews(lang='en', period='14d') # Specify period for news
        googlenews.get_news(f'${self.ticker} stock')
        stock_news = googlenews.results()
  
        # print([(i, j) for i, j in zip(googlenews.get_texts(), googlenews.get_links())])
        # To get other pages, do googlenews.get_page(2), etc.
  
        # Have whitelist of websites to search articles from. Maybe have key word to filter out stupid stuff.
  
        sectors = self.find_competition()
        sector_news = []
        if sectors:
            for sector in sectors:
                googlenews = GoogleNews(lang='en', period='14d')
                googlenews.get_news(f'{sector} sector stocks')
                sector_news.append(googlenews.result())
    
        return df, news, press_releases, sector_news, stock_news

    def financials(self): # OMEGALUL
        # Displaying all information. Could leave this as a dictionary.
        BASE_URL = f'https://finviz.com/quote.ashx?t={self.ticker}'
        soup = self._get_soup(BASE_URL)
        table = soup.find('table', {'class': 'snapshot-table2'})
        labels = table.find_all('td', {'class': 'snapshot-td2-cp'})
        values = table.find_all('td', {'class': 'snapshot-td2'})
        info_dict = {}
        for label, val in zip(labels, values):
            info_dict[str(label.get_text())] = str(val.get_text()) 
   
        info_dict['company_name'] = soup.find_all('b')[5].get_text()
        df = pd.DataFrame(info_dict.items(), columns={'Label', 'Value'})

        return df

    def short_selling(self):
        '''
        Returns a stocks short_selling information
        '''
        BASE_URL = f'https://finviz.com/quote.ashx?t={self.ticker}'
        soup = self._get_soup(BASE_URL)

        labels = soup.find_all('td', {'class': 'snapshot-td2-cp'})
        values = soup.find_all('td', {'class': 'snapshot-td2'})

        return labels[16].get_text(), values[16].get_text(), labels[22].get_text(), values[22].get_text()


    def put_call_ratio(self): 
        '''
        Returns various information regarding the put call ratio of a stock.
        '''
        BASE_URL = f'https://www.alphaquery.com/stock/{self.ticker}/volatility-option-statistics/120-day/put-call-ratio-oi'
        soup = self._get_soup(BASE_URL)

        ratio_volume = soup.find('tr', {'id': 'indicator-put-call-ratio-volume'})
        ratio_open_interest = soup.find('tr', {'id': 'indicator-put-call-ratio-oi'})
        forward_price = soup.find('tr', {'id': 'indicator-forward-price'})
        call_breakeven_price = soup.find('tr', {'id': 'indicator-call-breakeven'})
        put_breakeven_price = soup.find('tr', {'id': 'indicator-put-breakeven'})
        option_breakeven_price = soup.find('tr', {'id': 'indicator-option-breakeven'})

        return ratio_volume, ratio_open_interest, forward_price, call_breakeven_price, put_breakeven_price, option_breakeven_price

    def find_competition(self):
        BASE_URL = f'https://finviz.com/quote.ashx?t={self.ticker}'
        soup = self._get_soup(BASE_URL)

        td = soup.find_all('td', {'class': 'fullview-links'})[1]
        sectors = td.find_all('a', {'class': 'tab-link'})
        # sector_urls = ([str('https://finviz.com/' + i['href']) for i in sectors])
        # for url in sector_urls: # Find stocks with similar P/E ratios and market cap, then track difference in performance
        # 	print(url)
  
        sectors = [sector.get_text() for sector in sectors]
        return sectors

    def etfs(self):
        BASE_URL = f'https://etfdb.com/stock/{self.ticker}/'
        soup = self._get_soup(BASE_URL)
        tbody = soup.find('tbody')
        rows = tbody.find_all('tr')
        rows = [[i.get_text() for i in row.find_all('td')] for row in rows]
        train_df = pd.DataFrame(rows, columns={'self.Ticker', 'ETF', 'ETF Category', 'Expense Ratio', 'Weighting'})
        return train_df

    def insider_trading(self):
        BASE_URL = f'https://finviz.com/quote.ashx?t={self.ticker}'
        soup = self._get_soup(BASE_URL)

        tr = soup.find_all('tr', {'class': "insider-sale-row-2"})
        return [i.get_text() for i in tr]

        BASE_URL = f'https://www.secform4.com/'
        # Will be hard to get a stock, think I have to use Selenium to type in search bar and search.

    def social_media_sentiment(self, num_of_tweets=50): # Also reddit sentiment, and twitter
        # Twitter
        load_dotenv()
        consumer_key = os.getenv('TWEEPY_KEY')
        consumer_secret = os.getenv('TWEEPY_SECRET_KEY')
        auth = tweepy.AppAuthHandler(consumer_key, consumer_secret)
        api = tweepy.API(auth, wait_on_rate_limit=True)
        tweets = []
        for i, tweet in enumerate(tweepy.Cursor(api.search, q=f'${self.ticker}', count=num_of_tweets).items(num_of_tweets)):
            tweets.append(i, tweet.text, tweet.author.screen_name, tweet.retweet_count, tweet.favorite_count, tweet.created_at)
   
        # Get sentiment from stocktwits, get a token later. symbol, trending, sectors, symbols, 
        # Symbol
        BASE_URL = f'https://api.stocktwits.com/api/2/streams/symbol/{self.ticker}.json'
        symbol_messages = [message['body'] for message in get(BASE_URL).json()['messages']]

        BASE_URL = 'https://api.stocktwits.com/api/2/streams/trending.json'
        trending_messages = [message['body'] for message in get(BASE_URL).json()['messages']]

        # https://api.stocktwits.com/api/2/streams/sectors/technology.json?access_token=<access_token>
  
        return tweets, symbol_messages, trending_messages

    # Stocks with three good days in a row or above 9sma for three days. Good rsi = either high break out, piss low, ~40 support.
    
    def adl(self):
        data = self.get_month_data()
        adl = 0.0
        for row in data.iterrows():
            adl += (((row['Close'] - row['Low']) - (['High'] - row['Close']))/(['High'] - row['Low'])) * row['Volume']     
        
        return adl
  
 
    def technical_analysis(self):
        pass

    def fundamental_analysis(self):
        pass

    def seasonality(self):
        pass

    def catalysts(self): # Returns date of showcases, FDA approvals, earnings, etc
        df_data = []
        # Earnings date: 
        BASE_URL = f'https://finance.yahoo.com/quote/{self.ticker}?p={self.ticker}&.tsrc=fin-srch'
        soup = self._get_soup(BASE_URL)
        
        # Convert to datetime
        earnings_date = soup.find('td', {'data-test': 'EARNINGS_DATE-value'})
        earnings_date = f'Next earnings date: {earnings_date.get_text()}'

        # FDA approvals
        BASE_URL = 'https://www.rttnews.com/corpinfo/fdacalendar.aspx'
        soup = self._get_soup(BASE_URL)

        company = soup.find_all('div', {'data-th': 'Company Name'})

        events = soup.find_all('div', {'data-th': 'Event'})

        outcome = soup.find_all('div', {'data-th': 'Outcome'})

        dates = soup.find_all('span', {'class': 'evntDate'})

        for i in range(len(company)):
            if outcome[i]:
                if not len(dates[i].split()) > 1:
                    date = datetime.strptime(dates[i].get_text(), '%m/%d/%Y')
                else:
                    date = datetime.strptime(dates[i].split()[1:].get_text(), '%b %Y')

                df_data.append([date, company[i].get_text(), events[i].get_text(), outcome[i].get_text()])
            else:
                df_data.append([company[i].get_text(), events[i].get_text(), outcome[i]])

        # FDA trials
        df = pd.DataFrame(df_data, columns=['Date', 'Company Name', 'Event', 'Outcome'])
        # ?PageNum=4 to ?PageNum=1
        return df

    def big_money(self): # Returns recent institutional investments in a stock, as well as the largest shareholders and mutual funds holding the stock
        BASE_URL = f'https://money.cnn.com/quote/shareholders/shareholders.html?symb={self.ticker}&subView=institutional'
        soup = self._get_soup(BASE_URL)

        # Top 10 Owners
        df_data = []
        table = soup.find('table', {'class': 'wsod_dataTable wsod_dataTableBig wsod_institutionalTop10'})
        rows = table.find_all('tr')[1:]
        for row in rows:
            data = row.find_all('td')
            df_data.append([i.get_text() for i in data])

        owners_df = pd.DataFrame(df_data, columns=['Stockholder', 'Stake', 'Shares owned', 'Total value($)', 'Shares bought / sold', 'Total change'])

        # Top 10 Mutual Funds Holding 
        table = soup.find_all('table', {'class': 'wsod_dataTable wsod_dataTableBig wsod_institutionalTop10'})[1]
        rows = table.find_all('tr')[1:]
        df_data = []
        for row in rows:
            data = row.find_all('td')
            df_data.append([i.get_text() for i in data])

        mutual_funds_df = pd.DataFrame(df_data, columns=['Stockholder', 'Stake', 'Shares owned', 'Total value($)', 'Shares bought / sold', 'Total change'])

        # Recent instutional investments, from a better source this time
        # Make sure sorted by most recent.
        BASE_URL = f'https://fintel.io/so/us/{self.ticker}'
        soup = self._get_soup(BASE_URL)
        table = soup.find('table', {'id': 'transactions'})
        rows = table.find_all('tr')
        df_data = []
        for row in rows[1:]:
            date, form, investor, _, opt, avgshareprice, shares, shareschanged, value, valuechanged, _, _, _ = [i.get_text() for i in row.find_all('td')]
            df_data.append([date, form, investor, opt, avgshareprice, shares, shareschanged, value, valuechanged])
        recent_purchases_df = pd.DataFrame(df_data, columns=['Date', 'Form', 'Investor', 'Opt', 'Avg Share Price',
            'Shares', 'Shares Changed (%)', 'Value ($1000)', 'Value Changed (%)'])
        recent_purchases_df = recent_purchases_df.set_index('Date').sort_index(ascending=False)

        return owners_df, mutual_funds_df, recent_purchases_df.tail()


    def r_and_d(self):
        BASE_URL = f'https://ycharts.com/companies/{self.ticker}/r_and_d_expense'
        soup = self._get_soup(BASE_URL)
        tables = soup.find_all('table', {'class': 'histDataTable'})
        _pattern = re.compile(r'\d{1,3}\.\d{2}M')
        all_dicts = []
        for table in tables:
            keys = [key.get_text() for key in table.find_all('td', {'class': 'col1'})]
            vals = [_find_match(_pattern, val.get_text()).group() for val in table.find_all('td', {'class': 'col2'})]
            data_dict = {key : val for key, val in zip(keys, vals)}
            all_dicts.append(data_dict)
        return all_dicts

    # Options links: https://www.optionsprofitcalculator.com, f'https://marketchameleon.com/Overview/{ticker}/IV/', Need to find stock api that has option chain info like robinhood (greeks, iv, etc, price)	
    
    def __str__(self):
        return self.ticker

    def __repr__(self):
        return self.ticker

print('Welcome to Py-Trading!')

if __name__ == '__main__':
    # You can use this to test code out without it being imported/ran 
    pass



from datetime import date, datetime
import pandas as pd 
# from pygame import mixer
from time import sleep, strftime, localtime, time

# At the start of the day, get the top earners of the day, pickle it, and keep running the searching thing every 90 seconds (meaning time.sleep(15))

# mixer.init()

pd.options.display.max_rows = 1200
pd.options.display.max_columns = 10


def alert_sound():
	regular_instrumental = mixer.music.load("C:/Users/JWcam/Desktop/NCT 127 - Regular (Instrumental).mp3")
	mixer.music.set_volume(0.7)
	mixer.music.play()
	sleep(6.85)
	mixer.music.stop()


def stock_tracker(portfolio):
	"""Input which stocks you want to track, and the function will display the current price, low, highest change percentage, and
	current change percentage."""
	t1 = time()
	info = [f"Time: {strftime('%I:%M:%S', localtime())}"]
	print('Time ' + strftime('%I:%M:%S', localtime()))
	print([(stock.ticker, stock.df) for stock in portfolio])
	for stock in portfolio:

		# alert_sound() Do sound if within 5% of price target. Or do it if stock up 10% from price you bought it at. 
		try:
			stock.update_stock()
			cur_stats = stock.df_month.iloc[-1]
			prev_stats = stock.df_month.iloc[-2]
			intra_day_stats = stock.df
			m1, m5, m15, m30 = intra_day_stats.iloc[-1:][['Volume', 'Close']], intra_day_stats.iloc[-5:][['Volume', 'Close']]/5, intra_day_stats.iloc[-15:][['Volume', 'Close']]/5, intra_day_stats.iloc[-30:][['Volume', 'Close']]/5
			delta_volume = cur_stats['Volume']/sum(stock.df_month.iloc[-30:]['Volume'])
		except:
			continue
		
		# print(sum(m5['Volume'])/5, 1.25 * sum(m30['Volume'])/30, sum(m5['Close'])/5, 1.05 * sum(m30['Close'])/30)
		current_percentage = ((cur_stats['Close'] - prev_stats['Close'])/prev_stats['Close'])*100
		high_percentage = ((cur_stats['High'] - prev_stats['Close'])/prev_stats['Close'])*100
		low_percentage = ((cur_stats['Low'] - prev_stats['Close'])/prev_stats['Close'])*100
		# info.append(f"{stock.ticker}: Current: ${cur_stats['Close']:.2f}, High: ${cur_stats['High']:.2f}, Low: ${cur_stats['Low']:.2f}"
		# 	+ '\n' + f"Current: {change_percentage:.2f}%" + '\n' + f"High: {((cur_stats['High'] - prev_stats['Close'])/prev_stats['Close'])*100:.2f}%" + '\n')

		print(f"{stock.ticker.upper()}: Current: ${cur_stats['Close']:.2f}, High: ${cur_stats['High']:.2f}, Low: ${cur_stats['Low']:.2f}"
			+ '\n' + f"Current: {current_percentage:.2f}%" + '\n' + f"High: {high_percentage:.2f}%" + '\n' + f'Low: {low_percentage:.2f}%' + '\n')


# Detect stocks crossing the 9 MA + RSI below 45. For 9 MA crossing, maybe wait for it to cross above and be greater than for one/two periods (sum(m2['Close']) > sum(m2['9_sma']))

def trending_stocks(portfolio, signals):
	"""The function will display the stock's ticker and relevant information when a pattern is detected"""
	"""I could potentially load the data 20-25s before displaying the stocks, would have to sleep() for less, and have a while 
	loop in the function. So you would still have the print function displaying the old data during processing. 
	Could make this easier by appending all of the strings that will be displayed. Then you can control when they are displayed.
	"""
	# Alerts if a stock triggers a variety of signals. The program displays both the ticker and the signal that was triggered.
	# stocks_to_display = [strftime('%I:%M:%S', localtime())]
	print(end='\n' * 2)
	print('Time ' + strftime('%I:%M:%S', localtime()))
	t1 = time()

	good_stocks = []

	# Maybe periodically update the hottest stocks (ten mins) 
	# Show new stocks that are hot rn (very recently percent_change increase)

	signal_list = []

	def detect_pennant(intra_day):
		first, second, third, last = intra_day.iloc[-48:-36], intra_day.iloc[-36:-24], intra_day.iloc[-24:-12], intra_day.iloc[-12:]
		deltas = [first.max()['High'] - first.min()['Low'], second.max()['High'] - second.min()['Low'], third.max()['High'] - third.min()['Low'], last.max()['High'] - last.min()['Low']]
		pattern = len([i for i in range(3) if deltas[i] > deltas[i+1]]) >= 2
		return pattern, deltas

	def spike_1ma():
		if m1.iloc[0]['Volume'] > (1.25 * sum(m15['Volume'])) and m1.iloc[0]['Close'] > (1.015 * sum(m15['Close'])):
			print(f'{stock.ticker}, Current Price: ${cur_stats["Close"]}, Current: {current_percentage:.2f}%, High: {high_percentage:.2f}%, Low: {low_percentage:.2f}%, 1m')
			good_stocks.append(stock)		

	def spike_2ma():
		if sum(m2['Volume']) > (1.25 * sum(m15['Volume'])) and sum(m2['Close']) > (1.015 * sum(m15['Close'])):
			print(f'{stock.ticker}, Current Price: ${cur_stats["Close"]}, Current: {current_percentage:.2f}%, High: {high_percentage:.2f}%, Low: {low_percentage:.2f}%, 2m')
			good_stocks.append(stock)

	def basing():
		basing = intra_day_stats[-50:].nsmallest(5, ['Low'])['Low'].sort_values(ascending=True)
		basing = [price for price in basing[1:] if basing[0]*1.01 >= price] 
		print('Basing: ', basing, stock.ticker)

	def cross_9ma():
		if m15.iloc[-3]['9_sma'] > m15.iloc[-3]['Close'] and m15.iloc[-2]['9_sma'] < m15.iloc[-2]['Close'] and m1.iloc[0]['9_sma'] < m1.iloc[0]['Close']:
			print(f'{stock.ticker}, Current Price: ${cur_stats["Close"]}, Current: {current_percentage:.2f}%, High: {high_percentage:.2f}%, Low: {low_percentage:.2f}%, 9_sma crossed and held')
			good_stocks.append(stock)		

	def double_bottom():
		# Double_bottom, maybe the values have to be 10 mins apart or more
		bottom = intra_day_stats[-50:].nsmallest(5, ['Low'])['Low'].sort_values(ascending=True)
		bottom = [price for price in bottom[1:] if bottom[0]*1.01 >= price] 
		print(stock.ticker, 'Bottom: ', bottom)		

	def double_top():
		top = intra_day_stats[-50:].nsmallest(10, ['Low'])['Low'].sort_values(ascending=False)
		top = [price for price in top[1:] if top[0] <= price*1.01]
		print(stock.ticker, 'Top: ', top)

	if '1ma' in signals:
		signal_list.append(spike_1ma)
	if '2ma' in signals:
		signal_list.append(spike_2ma)
	if '9ma_cross' in signals:
		signal_list.append(cross_9ma)
	if 'double_bottom' in signals:
		signal_list.append(double_bottom)
	if 'double_top' in signals:
		signal_list.append(double_top)
	if 'basing' in signals:
		signal_list.append(basing)

	print([func.__name__ for func in signal_list])
	print([(stock.ticker, stock.df) for stock in portfolio])

	for stock in portfolio:
		try:
			stock.update_stock()
			intra_day_stats = stock.df
			m1, m2, m15, m30 = intra_day_stats.iloc[-1:][['Volume', 'Close', '2_sma', '9_sma']], intra_day_stats.iloc[-2:][['Volume', 'Close', '2_sma', '9_sma']]/2, intra_day_stats.iloc[-15:][['Volume', 'Close', '2_sma', '9_sma']]/15, intra_day_stats.iloc[-30:][['Volume', 'Close', '2_sma', '9_sma']]/30
			cur_stats = stock.df_month.iloc[-1]
			prev_stats = stock.df_month.iloc[-2]
			current_percentage = ((cur_stats['Close'] - prev_stats['Close'])/prev_stats['Close'])*100
			high_percentage = ((cur_stats['High'] - prev_stats['Close'])/prev_stats['Close'])*100
			low_percentage = ((cur_stats['Low'] - prev_stats['Close'])/prev_stats['Close'])*100
		except:
			continue

		# Maybe detect a drop in value as well?
		# print('30 min high', stock.ticker, m30.max(), m30.idxmax())
		# print(m1['9_sma'])


		# if len(bottom) > 0:
		# 	print(f'{stock.ticker} bottom @ {bottom[0]}')

		# General uptrend: can track this through 50 SMA crossing 200 SMA/100

		# OR COULID DO PUT ALL OF THE DESIRED FUNCTIONS IN A LIST AND LOOP THROUGH THEM FOR EVERY STOCK. CHOOSE WHICH FUNCTIONS
		# BY DOING THIS IF STATEMENT AT THE START 
		if cur_stats['Volume'] > 100_000:
			for signal in signal_list:
				signal()
			

	print(f'Computing time: {time() - t1:.2f}s. Average of {len(portfolio)/(time() - t1):.2f} stocks per second.')	

	return good_stocks




from datetime import date
import requests
import yfinance as yf 

# notify when near support/resistance levels
# Change resolution: 1, 5, 15, 30, 60, D, W, M
# Either just put all the functions in here, or can make a class where you put in a ticker then you get all the methods for that ticker


# Functions/Classes I prob won't use/update

# class StockTechnicals:

# 	def __init__(self, ticker, initial_price=None, shares=None):
# 		self.ticker = ticker
# 		self.api_key = 'bs2a4ffrh5rc90r54hm0'
# 		self.initial_price = initial_price
# 		self.shares = shares

# 	def get_aggregate_indicators(self):
# 		r = requests.get(f'https://finnhub.io/api/v1/scan/technical-indicator?symbol={self.ticker}&resolution=D&token={self.api_key}')
# 		response = r.json()['technicalAnalysis']
# 		return response

# 	def get_all_methods(self):
# 		# Could so self.get_aggregate_indictators.__name__, but that is extra
# 		return ['get_aggregate_indicators', 'get_company_news', 'get_live_price', 'get_news_sentiment', 'get_patterns', 
# 		'get_price_target', 'get_support_resistance', 'get_technical_indictator']

# 	def get_current_price(self):
# 		r = requests.get(f'https://finnhub.io/api/v1/company-news?symbol={self.ticker}&from={start_date}&to={end_date}&token={self.api_key}')
# 		response = r.json()
# 		return response['c']

# 	def get_company_news(self, start_date, end_date):
# 		r = requests.get(f'https://finnhub.io/api/v1/company-news?symbol={self.ticker}&from={start_date}&to={end_date}&token={self.api_key}')
# 		response = r.json()
# 		return response

# 	def get_live_price(self):
# 		r = requests.get(f'https://finnhub.io/api/v1/quote?symbol={self.ticker}&token={self.api_key}')
# 		response = r.json()
# 		return f"Current Price: ${response['c']}, Change %: {((response['c'] - response['pc'])/response['pc'])*100:.2f}%, High: ${response['h']}, Low: ${response['l']}, Open: ${response['o']}"
	
# 	def get_news_sentiment(self):
# 		r = requests.get(f'https://finnhub.io/api/v1/news-sentiment?symbol={self.ticker}&token={self.api_key}')
# 		response = r.json()
# 		return response

# 	def get_patterns(self):
# 		r = requests.get(f'https://finnhub.io/api/v1/scan/pattern?symbol={self.ticker}&resolution=D&token={self.api_key}')
# 		response = r.json()
# 		return response

# 	def get_price_target(self):
# 		r = requests.get(f'https://finnhub.io/api/v1/stock/price-target?symbol={self.ticker}&token={self.api_key}')
# 		response = r.json()
# 		return response

# 	def get_profit(self):
# 		initial_capital = self.initial_price * self.shares
# 		current_capital = self.get_current_price() * self.shares
# 		return current_capital - initial_capital

# 	def get_support_resistance(self):
# 		r = requests.get(f'https://finnhub.io/api/v1/scan/support-resistance?symbol={self.ticker}&resolution=D&token={self.api_key}')
# 		response = r.json()
# 		return response

# 	def get_technical_indictator(self, ticker, indicator):
# 		r = requests.get(f'https://finnhub.io/api/v1/indicator?symbol={self.ticker}&resolution=D&from=1583098857&to=1584308457&indicator={indicator}&timeperiod=3&token={self.api_key}')
# 		response = r.json()
# 		return response


# def check_basing(df):
# 	for i in range(5, df.shape[0]):
# 		prices = sorted(df.iloc[i-5:i]['Close'])
# 		change_percentage = ((prices[-1] - prices[0]) / prices[0])*100
# 		# Should I use the variance thing
# 		if change_percentage < 1.5:
# 			# df.iloc[i-5:i]
# 			print(f'{change_percentage:.2f}% High: {prices[0]:.2f} Low: {prices[-1]:.2f}', df.iloc[i])

# def get_support_levels(df):
# 	# Look at lows 10 mins late, so you can see if it is a real low or not
# 	for i in range(20, df.shape[0]):
# 		if len([price for price in df.iloc[i-20:i]['Low'] if price >= df.iloc[i-10]['Low']]) == 20:
# 			print('Support level', df.iloc[i-10])

# def get_current_trades(ticker):

# 	def on_message(ws, message):
# 	    print(message)

# 	def on_error(ws, error):
# 	    print(error)

# 	def on_close(ws):
# 	    print("### closed ###")

# 	def on_open(ws):
# 	    ws.send('{"type":"subscribe","symbol":"BILI"}')

# 	if __name__ == "__main__":
# 	    websocket.enableTrace(True)
# 	    ws = websocket.WebSocketApp(f"wss://ws.finnhub.io?token={api_key}",
# 	                              on_message = on_message,
# 	                              on_error = on_error,
# 	                              on_close = on_close)
# 	    ws.on_open = on_open
# 	    ws.run_forever()


# def get_rsi_levels(portfolio):
# 	for stock in portfolio:
# 		up_moves = []
# 		down_moves = []
# 		prices_14 = stock.df_month.iloc[-15:]
# 		for i in range(len(prices_14))[1:]:
# 			price_mvmt = prices_14.iloc[i]['Close'] - prices_14.iloc[i-1]['Close']
# 			if price_mvmt >= 0:
# 				up_moves.append(price_mvmt)
# 			else:
# 				down_moves.append(price_mvmt)
# 		try:
# 			up_avg = sum(up_moves)/len(up_moves)
# 			down_avg = abs(sum(down_moves)/len(down_moves))
# 		except:
# 			print('RSI of 0 or 100')
# 			continue
# 		rs = up_avg/down_avg
# 		print(stock.ticker, up_avg, down_avg, rs)
# 		rsi = 100 - 100/(1 + rs)
# 		print(stock.ticker, rsi)
		
# def double_bottom(df):
# 	current_low = df.iloc[0]['Low']
# 	for i in range(1, df.shape[0]):
# 		last_low =  df.iloc[i]['Low']
# 		prev_low = df.iloc[i-1]['Low']
# 		if last_low < current_low*1.005:
# 			if last_low > current_low*0.995:
# 				print('Double bottom', df.iloc[i])
# 			else:
# 				current_low = last_low
# 			print('New low', current_low, df.iloc[i])


# def dip_detector(df):
# 	falling = (False, None)
# 	for i in range(1, df.shape[0]):
# 		last_close =  last_close
# 		prev_close = df.iloc[i-1]['Close']
# 		if last_close < prev_close:
# 			falling = (True, prev_close)
# 		elif last_close > prev_close:
# 			if df.iloc[i] >= falling[1]:
# 				falling = (False, None)
# 			elif falling:
# 				print('Basing')
# 				falling = (False, falling[1])
# 			print(df.iloc[i], df.iloc[i-1])


# def sma_cross(df):
# 	for i in range(14, df.shape[0]):
# 		if (df.iloc[i-1]['8_sma'] < df.iloc[i-1]['13_sma']) and (df.iloc[i]['8_sma'] > df.iloc[i]['13_sma']):
# 			print('8 SMA CROSSED 13 SMA!!!', df.iloc[i-1]['8_sma'], df.iloc[i-1]['13_sma'], df.iloc[i]['8_sma'], df.iloc[i]['13_sma'])
# 			print(df.iloc[i])	
# 		elif (df.iloc[i-1]['8_sma'] > df.iloc[i-1]['13_sma']) and (df.iloc[i]['8_sma'] < df.iloc[i]['13_sma']):
# 			print('13 SMA CROSSED 8 SMA!!!', df.iloc[i-1]['8_sma'], df.iloc[i-1]['13_sma'], df.iloc[i]['8_sma'], df.iloc[i]['13_sma'])
# 			print(df.iloc[i])
# 		if df.iloc[i]['8_sma'] > df.iloc[i]['Close']:
# 			print(df.iloc[i])


# def support(df):
# 	support_prices = []
# 	for i in range(1, df.shape[0]):
# 		cur_low, last_low = get_lows(df)
		# If the somewhere in the next 10 mins the price is 1-2% higher than here, counts as low. And currently lower than the last_low
		# if cur_low < last_low and all([price for price in df.iloc[i:i+5] if price > next_low or not ]):


# Check if webull charts agree with how much it crosses, maybe I need to make error larger?
# Dip detector. Counts as a dip when every 1m close is lower, but then once the next close is higher, that's the bottom
# What if I made a machine learning algorithm where I fed it stock prices during a dip. It comes up with a parabolic function?




from requests import get 
from bs4 import BeautifulSoup
from datetime import datetime
import pickle
from pathlib import Path
import pandas as pd

# IMPLEMENT THREADING
# results = [executor.submit(test_stocks, i, n_threads) for i in range(n_threads)] # Can try executor.map()
        
#         for f in concurrent.futures.as_completed(results):
#             print(f.result())
    
# def test_stocks(index_of_thread, num_of_threads): # Divide # of stocks per thread / total stocks to be tested. Index_of_thread is which thread from 0 to n threads.
#     n_stocks_per_thread = len(Stock.objects.all()) 
#     portion = Stock.objects.all()[index_of_thread*n_stocks_per_thread:(index_of_thread+1)*n_stocks_per_thread]

def get_sp500():
	request = get('https://en.wikipedia.org/wiki/List_of_S%26P_500_companies')
	soup = BeautifulSoup(request.text, 'lxml')
	table = soup.find('table')
	df = pd.read_html(str(table))
	return df[0]

def get_nasdaq(): # Nasdaq + NYSE + AMEX
    dfs = []
    for letter in 'abcdefghijklmnopqrstuvwxyz':
        request = get(f'https://www.advfn.com/nasdaq/nasdaq.asp?companies={letter.upper()}')
        soup = BeautifulSoup(request.text, 'lxml')
        table = soup.find('table', {'class': 'market tab1'})
        df = pd.read_html(str(table))[0]
        df.columns = df.iloc[1].tolist()
        df = df.iloc[2:]
        df = df.reset_index()
        df = df[['Symbol', 'Equity']]
        df.columns = ['ticker', 'name']
        dfs.append(df)
        
    for letter in 'abcdefghijklmnopqrstuvwxyz':           
        request = get(f'http://eoddata.com/stocklist/NASDAQ/{letter}.htm')
        soup = BeautifulSoup(request.text, 'lxml')
        table = soup.find('table', {'class': 'quotes'})
        df = pd.read_html(str(table))[0]
        df = df[['Code', 'Name']]
        df.columns = ['ticker', 'name']
        dfs.append(df)
  
    df = pd.concat(dfs)
    df = df.reset_index()
    df = df[['ticker', 'name']]
    # if as_list:
        # return df.set_index('ticker').to_dict()
    return df

def get_nyse(): # Test to see if duplicate tickers on backend or Django webapp
    dfs = []
    for letter in 'abcdefghijklmnopqrstuvwxyz':
        request = get(f'https://www.advfn.com/nyse/newyorkstockexchange.asp?companies={letter.upper()}')
        soup = BeautifulSoup(request.text, 'lxml')
        table = soup.find('table', {'class': 'market tab1'})
        df = pd.read_html(str(table))[0]
        df.columns = df.iloc[1].tolist()
        df = df.iloc[2:]
        df = df.reset_index()
        df = df[['Symbol', 'Equity']]
        df.columns = ['ticker', 'name']
        dfs.append(df)
        
    for letter in 'abcdefghijklmnopqrstuvwxyz':       
        request = get(f'https://eoddata.com/stocklist/NYSE/{letter}.htm')
        soup = BeautifulSoup(request.text, 'lxml')
        table = soup.find('table', {'class': 'quotes'})
        try:
            df = pd.read_html(str(table))[0]
        except:       
            df = pd.read_html(str(table))            
        df = df[['Code', 'Name']]
        df.columns = ['ticker', 'name']
        dfs.append(df)
        
    	# Will this work since they are series?
    df = pd.concat(dfs)
    df = df.reset_index()
    df = df[['ticker', 'name']]
    # df['ticker'] = df['ticker'].unique()
    # df['name'] = df['name'].unique()
    # if as_list:
    #     return sorted(df.tolist())
    return df.sort_values(by='ticker', ascending=True)


# def get_biggest_movers():
# 	tickers = []
# 	request = get('https://www.tradingview.com/markets/stocks-usa/market-movers-gainers/')
# 	soup = BeautifulSoup(request.text, 'lxml')
# 	table = soup.find('tbody', {'class': 'tv-data-table__tbody'})
# 	for i in table.find_all('a', {'class': 'tv-screener__symbol'})[::2]:
# 		tickers.append(i.get_text())

# 	request = get('http://thestockmarketwatch.com/markets/topstocks/')
# 	soup = BeautifulSoup(request.text, 'lxml')
# 	table = soup.find_all('div', {'class': 'activestockstbl'})

# 	return list(set(tickers))


def get_day_hot_stocks():
	url = 'https://www.tradingview.com/markets/stocks-usa/market-movers-gainers/'
	page = get(url)
	soup = BeautifulSoup(page.content, 'html.parser')
	rows = soup.find_all('tr', {'class':'tv-data-table__row tv-data-table__stroke tv-screener-table__result-row'})
	return [row.find('a').get_text() for row in rows]


def get_day_premarket_movers():
	url = 'https://thestockmarketwatch.com/markets/pre-market/today.aspx'
	page = get(url)
	soup = BeautifulSoup(page.content, 'html.parser')
	table = soup.find_all('table', {'id': 'tblMoversDesktop'})[0]
	try:
		print('Biggest winners from TheStockMarketWatch:')
		marketwatch_list = [(ticker.get_text(), float(change.get_text()[:-1].replace(',',''))) for ticker, change in zip(table.find_all('td', {'class': 'tdSymbol'}), table.find_all('td', {'class': 'tdChangePct'}))]
		for ticker, percentage in sorted(marketwatch_list, key=lambda x: x[1], reverse=True):
			print(f'{ticker}: {percentage}%')
	except:
		print('Due to unseen errors, the stockmarketwatch list is unable to be reached.')
	print()

	url = 'https://www.benzinga.com/money/premarket-movers/'
	page = get(url)
	soup = BeautifulSoup(page.content, 'html.parser')
	div = soup.find('div', {'id': 'movers-stocks-table-gainers'})
	tbody = div.find('tbody')
	data = [(i.get_text().replace('\n    ', '')[2:], float(j.get_text().replace('\n  ', '')[2:-1])) for i, j in zip(tbody.find_all('a', {'class': 'font-normal'}), tbody.find_all('td')[3::5])]
	try:
		print('Biggest winners from Benzinga:')
		for ticker, percentage in data:
			print(f'{ticker}: {percentage}%')
	except:
		print('Due to unseen errors, the Benzinga list is unable to be reached.')

def get_silver_stocks():
	url = 'http://www.24hgold.com/english/listcompanies.aspx?fundamental=datas&data=company&commodity=ag&commodityname=SILVER&sort=resources&iordre=107'
	page = get(url)
	soup = BeautifulSoup(page.content, 'html.parser')
	table = soup.find('table', {'id': 'ctl00_BodyContent_tbDataExport'})
	rows = table.find_all('td', {'class': ['cell_bleue_center', 'cell_gold_right']})
	for i in range(len(rows))[28::2]:
		if not '.' in rows[i].get_text():
			print(f'{rows[i].get_text()}: ${rows[i+1].get_text()}')


def load_biggest_movers():
	path = Path(__file__).parents[1]
	with open(f'{path}/dailypickle/biggest_movers-{datetime.now().strftime("%m-%d-%Y")}.pkl', 'rb') as f:
		return pickle.load(f)


def pickle_biggest_movers(portfolio):
	path = Path(__file__).parents[1]
	with open(f'{path}/dailypickle/biggest_movers-{datetime.now().strftime("%m-%d-%Y")}.pkl', 'wb') as f:
		pickle.dump(portfolio, f)

def load_positions():
	path = Path(__file__).parents[1]
	with open(f'{path}/dailypickle/positions-{datetime.now().strftime("%m-%d-%Y")}.pkl', 'rb') as f:
		return pickle.load(f)


def pickle_positions(portfolio):
	# This is useless
	path = Path(__file__).parents[1]
	with open(f'{path}/dailypickle/positions-{datetime.now().strftime("%m-%d-%Y")}.pkl', 'wb') as f:
		pickle.dump(portfolio, f)


def pickle_dump(portfolio):
	today = date.today()
	with open(f"{today.strftime('%m-%d')}_pickle.pkl", 'wb') as f:
		pickle.dump(portfolio, f)


def pickle_load():
	today = date.today()
	with open(f"{today.strftime('%m-%d')}_pickle.pkl", 'rb') as f:
		return pickle.load(f)




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Py-Trading",  # Replace with your own username
    version="0.4.7.5",
    author="Joseph Camyre",
    author_email="jwcamry03@gmail.com",
    description="A convenient Python module for stock information.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/JCamyre/Python-Trading",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'asgiref==3.3.2',
        'beautifulsoup4==4.9.3',
        'certifi==2020.12.5',
        'cffi==1.14.4',
        'chardet==4.0.0',
        'click==7.1.2',
        'cycler==0.10.0',
        'et-xmlfile==1.0.1',
        'Flask==1.1.2',
        'GoogleNews==1.5.5',
        'html5lib==1.1',
        'idna==2.10',
        'itsdangerous==1.1.0',
        'jdcal==1.4.1',
        'Jinja2==2.11.3',
        'kiwisolver==1.3.1',
        'lxml>=4.6.3',
        'MarkupSafe==1.1.1',
        'matplotlib==3.3.3',
        'mplfinance==0.12.7a5',
        'numpy==1.19.5',
        'oauthlib==3.1.0',
        'openpyxl==3.0.6',
        'pandas>=1.15.1',
        'Pillow>=8.2.0',
        'pycparser==2.20',
        'pyOpenSSL==20.0.1',
        'pyparsing==2.4.7',
        'python-dateutil==2.8.1',
        'python-dotenv==0.15.0',
        'pytz==2020.5',
        'requests==2.25.1',
        'requests-oauthlib==1.3.0',
        'six==1.15.0',
        'tweepy==3.10.0',
        'urllib3>=1.26.5',
        'websockets>=9.1',
        'Werkzeug==1.0.1',
    ]
)
